<?php
/*
 * ARI Slick Gallery
 *
 * @package		ARI Slick Gallery
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

jimport('joomla.filesystem.file');
jimport('joomla.filter.filterinput');

AriKernel::import('Utils.Utils');
AriKernel::import('Utils.AppUtils');
AriKernel::import('FileSystem.Folder');

class AriSlickGalleryHelper
{
	static public function includeAssets($params)
	{
		static $loaded;

		if ($loaded)
			return ;
			
		$loadJQuery = (bool)$params->get('includeJQuery', true);
		$noConflict = (bool)$params->get('noConflict', true);
		$theme = $params->get('theme', '');

		$doc = JFactory::getDocument();
		$baseUri = JURI::root(true) . '/modules/mod_arislickgallery/mod_arislickgallery/js/';
		if ($loadJQuery) 
		{
            if (J3_0)
                JHtml::_('jquery.framework', $noConflict);
            else
            {
			    $doc->addScript('http://ajax.googleapis.com/ajax/libs/jquery/1.4.4/jquery.min.js');

			    if ($noConflict)
    				$doc->addScript($baseUri . 'jquery.noconflict.js');
			}
		}

		//$doc->addScript('http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.4/jquery-ui.min.js');
		$doc->addScript($baseUri . 'jquery-ui.js');
		$doc->addScript($baseUri . 'script.js');

		$theme = JFilterInput::getInstance()->clean($theme, 'WORD');
		if (empty($theme)) $theme = 'default';

		$doc->addStyleSheet($baseUri . 'themes/' . $theme . '/style.css');

		$loaded = true;
	}
	
	public static function initGallery($id, $params)
	{
		self::includeAssets($params);

		$clientParams = self::getClientParams($params);
		$doc = JFactory::getDocument();
		
		$doc->addScriptDeclaration(
			sprintf('jQuery(window).load(function() { var $ = window.jQuerySlickGallery || jQuery; $("#%1$s").ariSlickGallery(%2$s); });',
				$id,
				$clientParams ? json_encode($clientParams, true) : '')
		);
		
		$width = intval($params->get('width', 300), 10);
		$height = intval($params->get('height'), 10);
		
		$styleDec = sprintf('#%1$s{width:%2$dpx;height:%3$dpx;}',
			$id,
			$width,
			$height);

		if ($params->get('customstyle'))
		{
			$extraStyles = trim($params->get('customstyle'));
			$extraStyles = str_replace('{$id}', '#' . $id, $extraStyles);
			if (!empty($extraStyles))
				$styleDec .= $extraStyles;
		}
		
		$doc->addStyleDeclaration($styleDec);
	}
	
	public static function getClientParams($params)
	{
		$clientParams = array(
			'hideTooltip' => false
		);
		
		$galleryParams = array();
		foreach ($clientParams as $key => $value)
		{
			$param = $params->get('opt_' . $key, null);
			if (is_null($param))
				continue ;
				
			$param = AriUtils::parseValueBySample($param, $value);
			if ($value !== $param)
				$galleryParams[$key] = $param;
		}
		
		return $galleryParams;
	}
	
	public static function getImages($params)
	{
		$slides = array();

		$pathList = self::getPathList($params->get('path'));
		if (count($pathList) == 0)
			return $slides;

		$extraFields = array();
		$i18n = (bool)$params->get('i18n', false);
		$descrFile = trim($params->get('descrFile', ''));
		$processDescrFile = !empty($descrFile);
		$processSubDir = (bool)$params->get('subdir');
		
		$imgNumber = intval($params->get('imgNumber', 0), 10);
		$images = array();
		$sort = self::getSortExpr($params);
		foreach ($pathList as $path)
		{
			if (empty($path))
				continue ;
			
			if (@is_file($path))
			{
				$images[] = $path;
				$path = dirname($path);
			}
			else
			{
				$folderImages = AriFolder::files(
					$path,
					'.(jpg|gif|jpeg|png|bmp|JPG|GIF|JPEG|BMP)$', 
					$processSubDir,
					true,
					$sort);

				if (is_array($folderImages) && count($folderImages) > 0)
				{
					if ($imgNumber > 0 && count($folderImages) > $imgNumber)
						$folderImages = array_slice($folderImages, 0, $imgNumber);

					$images = array_merge($images, $folderImages);
				}
			}
			
			if ($processDescrFile)
			{
				$folderExtraFields = AriAppUtils::getExtraFieldsFromINI($path, $descrFile, $processSubDir, true, $i18n);
				if (is_array($folderExtraFields) && count($folderExtraFields) > 0)
				{				
					$extraFields = array_merge($extraFields, $folderExtraFields);
				}
			}
		}
		
		$useThumbs = (bool)$params->get('imglist_useThumbs');
		$cachePath = $useThumbs ? self::getCachePath() : null;
		$thumbPath = $params->get('imglist_thumbPath');
		$thumbWidth = intval($params->get('imglist_thumbWidth'), 10);
		$thumbHeight = intval($params->get('imglist_thumbHeight'), 10);
		foreach ($images as $image)
		{
			$slide = array(
				'src' => $image,
				'image' => str_replace('\\', '/', $image)
			);
			
			if ($processDescrFile && isset($extraFields[$image]))
				$slide = array_merge($extraFields[$image], $slide);
				
			if ($useThumbs)
				$slide = self::generateThumbnail($slide, $thumbWidth, $thumbHeight, $thumbPath, $cachePath);

			$slides[] = $slide;
		}

		return $slides;
	}

	public static function prepareImages($slides, $params, $modId)
	{
		$newSlides = array();
		
		$target = $params->get('linkTarget', '_self');
		$baseUri = JURI::base(true);
		$lightboxEngine = self::getLightboxEngine($params);
		$lightboxGroup = uniqid('cc_');
		$width = intval($params->get('width', 300), 10);
		$height = intval($params->get('height'), 10);
		$startDegree = intval($params->get('gallery_startDegree', -40));
		$endDegree = intval($params->get('gallery_endDegree', 40));
		$operaSupport = (bool)$params->get('gallery_operaSupport', false);
		$randomPos = (bool)$params->get('gallery_randomPos', true);
		$coordinates = !$randomPos ? self::getCachedCoordinates($modId) : array();
		if (!is_array($coordinates))
			$coordinates = array();
			
		$prevItemsCount = count($coordinates);

		foreach ($slides as $slide)
		{
			$isLink = !empty($slide['link']);
			$description = isset($slide['description']) ? $slide['description'] : '';
			
			$slideWidth = null;
			$slideHeight = null;
			if (empty($slide['width']) || empty($slide['height']))
			{
				
				if ($size = @getimagesize(JPATH_ROOT . '/' . $slide['image']))
				{
					$slideWidth = $size[0];
					$slideHeight = $size[1];
				}
			}
			else
			{
				$slideWidth = $slide['width'];
				$slideHeight = $slide['height'];
			}
		
			$lnkAttrs = null;
			$left = 0;
        	$top = 0;
        	$rotateDegree = 0;
        	
        	if (!$randomPos && isset($coordinates[$slide['src']]))
        	{
        		$coordItem = $coordinates[$slide['src']];
        		
        		$left = $coordItem['left'];
        		$top = $coordItem['top'];
        		$rotateDegree = $coordItem['degree'];
        	}
        	else
        	{
        		$left = rand(0, $width - ($slideWidth ? 1.42 * $slideWidth : 0));
        		$top = rand(0, $height - ($slideHeight ? 1.42 * $slideHeight : 0));
        		$rotateDegree = rand($startDegree, $endDegree);
        		
        		if (!$randomPos)
        			$coordinates[$slide['src']] = array(
        				'left' => $left,
        				'top' => $top,
        				'degree' => $rotateDegree
        			);
        	}

			$imgAttrs = array('style' => 
				array(
					'background-image' => 'url(\'' . $baseUri . '/' . $slide['image'] . '\')',
					'top' => $top . 'px',
					'left' => $left . 'px',
					'-moz-transform' => 'rotate(' . $rotateDegree . 'deg)',
					'-webkit-transform' => 'rotate(' . $rotateDegree . 'deg)',
					'transform' => 'rotate(' . $rotateDegree . 'deg)'
				), 
				'class' => 'arislickgallery-item'
			);
			
			if ($operaSupport) $imgAttrs['style']['-o-transform'] = 'rotate(' . $rotateDegree . 'deg)';

			$lnkAttrs = array('href' => $isLink ? $slide['link'] : 'javascript:void(0)', 'target' => $isLink ? $target : '_self', 'style' => array());
			if ($description)
				$lnkAttrs['title'] = $description;
					
			if ($isLink && !is_null($lightboxEngine))
				list($lnkAttrs, $imgAttrs) = $lightboxEngine->modifyAttrs($lnkAttrs, $imgAttrs, $lightboxGroup, $params);
					
			if ($slideWidth) 
				$imgAttrs['style']['width'] = $lnkAttrs['style']['width'] = $slideWidth . 'px';
			if ($slideHeight) 
				$imgAttrs['style']['height'] = $lnkAttrs['style']['height'] = $slideHeight . 'px';

			$slide['lnkAttrs'] = $lnkAttrs;
			$slide['imgAttrs'] = $imgAttrs;
			$newSlides[] = $slide;
		}
		
		if (!$randomPos && $prevItemsCount != count($coordinates))
			self::cacheCoordinates($modId, $coordinates);

		return $newSlides;
	}

	public static function generateThumbnail($slide, $thumbWidth, $thumbHeight, $thumbPath, $cachePath)
	{
		$img = $slide['src'];
		$imgUri = $slide['image'];
		$thumbFile = null;
		$autoThumb = true;
		
		if ($thumbPath)
		{
			$pathInfo = pathinfo($img);
			$thumbImg = $pathInfo['dirname'] . '/' .  str_replace('{$fileName}', $pathInfo['basename'], $thumbPath);
			if (@file_exists(JPATH_ROOT . '/' . $thumbImg) && @is_file(JPATH_ROOT . '/' . $thumbImg))
			{
				$thumbFile = $thumbImg;
				$autoThumb = false;
			}
		}

		if (is_null($thumbFile))
		{
			$thumbName = AriImageHelper::generateThumbnail(
				JPATH_ROOT . '/' . $img,
				JPATH_ROOT . '/' . $cachePath,
				'ais',
				$thumbWidth,
				$thumbHeight);
			if ($thumbName)
			{
				$thumbFile = $cachePath . '/' . $thumbName;
			}
			else
			{
				$thumbFile = $thumbImg;
			}
		}

		$slide['src'] = $thumbFile;
		$slide['image'] = str_replace('\\', '/', $thumbFile);
		if (empty($slide['link']))
			$slide['link'] = $imgUri;
			
		$thumbSize = array('w' => $thumbWidth, 'h' => $thumbHeight);
		if ($autoThumb)
		{
			$thumbSize = AriImageHelper::getThumbnailDimension(JPATH_ROOT . '/' . $thumbFile, $thumbWidth, $thumbHeight);
		}
		else
		{
			$size = @getimagesize(JPATH_ROOT . '/' . $thumbFile);
			if (!empty($size))
				$thumbSize = array('w' => $size[0], 'h' => $size[1]);
		}

		if (!empty($thumbSize['w'])) $slide['width'] = $thumbSize['w'];
		if (!empty($thumbSize['h'])) $slide['height'] = $thumbSize['h'];
		
		return $slide;
	}

    public static function getCachePath()
	{
		$cacheDir = 'images';
		$extCacheDir = $cacheDir . '/mod_arislickgallery';
		if (@file_exists($extCacheDir) && is_dir($extCacheDir))
		{
			$cacheDir = $extCacheDir;
		}
		
		return $cacheDir;
	}

    public static function getCachedCoordinates($modId)
	{
		$coordinates = null;
		$file = self::getCachePath() . '/' . self::getCacheCoordinatesFileName($modId);
		if (!@file_exists($file) || !@is_readable($file))
			return $coordinates;

		$cachedCoordinates = file_get_contents($file);
		if ($cachedCoordinates)
			$coordinates = json_decode($cachedCoordinates);

		return $coordinates;
	}

    public static function cacheCoordinates($modId, $coordinates)
	{		
		$file = self::getCachePath() . '/' . self::getCacheCoordinatesFileName($modId);
		
		if (empty($coordinates) || (is_array($coordinates) && count($coordinates) == 0))
			return JFile::delete($file);
		else
			return JFile::write($file, json_encode($coordinates, true));
	}

    public static function getCacheCoordinatesFileName($modId)
	{
		return 'sg_' . $modId . '_coordinates';
	}

    public static function getSortExpr($params)
	{
		$sortBy = $params->get('sortBy');
		if (empty($sortBy) || !in_array($sortBy, array('filename', 'modified', 'random')))
			return null;
			
		return array(
			'sortBy' => $sortBy, 
			'sortDir' => ($params->get('sortDir') == 'desc' ? 'desc' : 'asc')
		);
	}

    public static function getPathList($path)
	{
		$pathList = array();
		if (empty($path))
			return $pathList;

		$pathList = explode("\n", $path);
		array_walk($pathList, array('AriFolder', 'clean'));
		$pathList = array_unique($pathList);
		
		return $pathList;
	}

    public static function getLightboxEngine($params)
	{
		$engine = null;
		$engineName = ucfirst(JFilterInput::getInstance()->clean($params->get('lightboxEngine'), 'WORD'));
		if (empty($engineName))
			return null;
		
		AriKernel::import('SlickGallery.Lightbox.' . $engineName);
		
		$className = $engineName . 'SlickGalleryEngine';
		if (class_exists($className))
		{
			$engine = new $className();
			if (!$engine->preCheck())
			{
				$engine = null;
			}
		}
		
		return $engine;
	}
}